Package: CoSMoS
Type: Package
Title: Complete Stochastic Modelling Solution
Version: 2.2.0
Date: 2026-05-07
Authors@R: c(person("Simon Michael", "Papalexiou", 
                      role = "aut"),
             person("Francesco", "Serinaldi", 
                      role = "aut"),
             person("Filip", "Strnad",
                     role = "aut"),
             person("Yannis", "Markonis", 
                      role = "aut"),
             person("Kevin", "Shook", 
                      role = c("ctb", "cre"),
                      email = "kshook@kshook.ca")
              )
Description: Makes univariate, multivariate, or random fields simulations precise and simple. Just select the desired time series or random fields’ properties and it will do the rest. CoSMoS is based on the framework described in Papalexiou (2018, <doi:10.1016/j.advwatres.2018.02.013>), extended for random fields in Papalexiou and Serinaldi (2020, <doi:10.1029/2019WR026331>), and further advanced in Papalexiou et al. (2021, <doi:10.1029/2020WR029466>) to allow fine-scale space-time simulation of storms (or even cyclone-mimicking fields).
Depends: R (>= 3.5.0), ggplot2, data.table
Imports: utils, methods, stats, grDevices, nloptr, MBA, Matrix, mAr,
        matrixcalc, mvtnorm, patchwork, animation, ggquiver, pracma,
        plot3D, Rcpp
LinkingTo: Rcpp, RcppEigen, RcppNumerical, BH
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Suggests: testthat, knitr, rmarkdown
VignetteBuilder: knitr
Author: Simon Michael Papalexiou [aut],
  Francesco Serinaldi [aut],
  Filip Strnad [aut],
  Yannis Markonis [aut],
  Kevin Shook [ctb, cre]
Maintainer: Kevin Shook <kshook@kshook.ca>
License: GPL-3
URL: https://github.com/TycheLab/CoSMoS
NeedsCompilation: yes
Packaged: 2026-05-07 18:24:04 UTC; kevin
Repository: CRAN
Date/Publication: 2026-05-07 20:02:51 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-07 22:29:00 UTC; unix
Archs: CoSMoS.so.dSYM
