# Generated by roxygen2: do not edit by hand

S3method(plot,acti)
S3method(plot,checkTS)
S3method(plot,cosmosts)
S3method(plot,fitACS)
S3method(plot,fitDist)
export(acs)
export(actf)
export(actfInv)
export(actfdiscrete)
export(acti)
export(actpnts)
export(actpntsB6)
export(advectionF)
export(advectionFhyperbolic)
export(advectionFradial)
export(advectionFrotation)
export(advectionFspiral)
export(advectionFspiralCE)
export(advectionFuniform)
export(analyzeTS)
export(anisotropyT)
export(anisotropyTaffine)
export(anisotropyTswirl)
export(anisotropyTwave)
export(checkRF)
export(checkTS)
export(dburrIII)
export(dburrXII)
export(dgev)
export(dggamma)
export(dparetoII)
export(fitACS)
export(fitDist)
export(fitVAR)
export(fitactf)
export(generateMTS)
export(generateMTSFast)
export(generateRF)
export(generateRFFast)
export(generateTS)
export(mburrIII)
export(mburrXII)
export(mgev)
export(mggamma)
export(moments)
export(mparetoII)
export(pburrIII)
export(pburrXII)
export(pgev)
export(pggamma)
export(populationstat)
export(pparetoII)
export(qburrIII)
export(qburrXII)
export(qgev)
export(qggamma)
export(qparetoII)
export(quickTSPlot)
export(rburrIII)
export(rburrXII)
export(regenerateTS)
export(reportTS)
export(rgev)
export(rggamma)
export(rparetoII)
export(sample.moments)
export(simulateTS)
export(stcfclayton)
export(stcfgneiting14)
export(stcfgneiting16)
export(stcs)
import(animation)
import(data.table)
import(ggplot2)
import(ggquiver)
import(graphics)
import(mAr)
import(methods)
import(mvtnorm)
import(nloptr)
import(patchwork)
import(plot3D)
import(stats)
import(utils)
importFrom(MBA,mba.surf)
importFrom(Matrix,nearPD)
importFrom(Rcpp,sourceCpp)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,n2mfrow)
importFrom(matrixcalc,is.positive.definite)
importFrom(pracma,distmat)
useDynLib(CoSMoS, .registration = TRUE)
