#' Daily streamflow data data
#'
#' Station details
#' * Name: Nassawango Creek near Snow Hill, Worcester County, Maryland, Hydrologic Unit 02080111
#' * Network Id: , USGS 01485500
#' * Latitude/Longitude: 38°13'44.1", 75°28'17.2"
#' * Elevation: 11.49 ft above North American Vertical Datum of 1988.
#' * Measurement unit: cubic feet per second
#'
#' more details can be found \href{https://waterdata.usgs.gov/monitoring-location/USGS-01485500/}{here}.
#'
#' @format A data.table with 23315 rows and 2 variables:
#' \describe{
#'   \item{date}{POSIXct format date/time}
#'   \item{value}{daily avarage values}
#' }
#' @source The United States Geological Survey (USGS) National Water Information System (NWIS)
"disch"
