# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Fast actpnts via nested C++ 1D integration
#'
#' Internal C++ implementation. Do not call directly.
#'
#' @param margdist character distribution name
#' @param margarg  named numeric vector of distribution parameters
#' @param p0       probability zero
#' @param lower    integration lower bound
#' @param upper    integration upper bound
#' @param mu1      first raw moment
#' @param mu2      second central moment
#' @param rhoz_vals numeric vector of Gaussian correlation values
#' @param max_eval  max evaluations per 1D integral (default 1e4)
#' @param abs_tol   absolute tolerance (default 1e-5)
#' @param rel_tol   relative tolerance (default 1e-5)
#'
#' @return data.frame with columns rhoz and rhox
#' @keywords internal
actpnts_cpp <- function(margdist, margarg, p0, lower, upper, mu1, mu2, rhoz_vals, max_eval = 10000L, abs_tol = 1e-5, rel_tol = 1e-5) {
    .Call(`_CoSMoS_actpnts_cpp`, margdist, margarg, p0, lower, upper, mu1, mu2, rhoz_vals, max_eval, abs_tol, rel_tol)
}

