% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/actf.R
\name{fitactf}
\alias{fitactf}
\title{Fit the AutoCorrelation Transformation Function}
\usage{
fitactf(actpnts, discrete = FALSE)
}
\arguments{
\item{actpnts}{estimated ACT points (output of \code{\link{actpnts}})}

\item{discrete}{logical — is the marginal distribution discrete?}
}
\value{
An object of class \code{"acti"} with components:
\describe{
\item{actfcoef}{fitted ACTF coefficients \code{b} and \code{c}}
\item{actfpoints}{the input ACT points data frame}
}
}
\description{
Fits the ACTF to the estimated (rho_x, rho_z) points using \code{nls}.
}
\examples{

library(CoSMoS)

p   <- actpnts(margdist = "paretoII",
               margarg  = list(scale = 1, shape = .3),
               p0 = 0)
fit <- fitactf(p)
plot(fit)

}
\seealso{
\code{\link{actpnts}}, \code{\link{actf}}
}
