% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ecdf.R, R/fitting.R
\name{ECDF}
\alias{ECDF}
\title{Empirical cumulative distribution function}
\usage{
ECDF(x)

ECDF(x)
}
\arguments{
\item{x}{numeric vector of values}
}
\value{
A \code{data.table} with columns \code{p} (plotting positions)
and \code{value} (sorted unique values).

a \code{data.table} with columns \code{p} (plotting positions) and
\code{value} (sorted unique values), keyed on \code{value}
}
\description{
Calculates the ECDF using the Weibull plotting position formula.

Computes the ECDF using the Weibull plotting position.
}
\seealso{
\code{\link{fitDist}}
}
\keyword{internal}
