% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/population.R
\name{PopulationStat}
\alias{PopulationStat}
\alias{populationstat}
\alias{popmean}
\alias{popsd}
\alias{popvar}
\alias{popcvar}
\alias{popskew}
\alias{popkurt}
\title{Population statistics}
\usage{
populationstat(stat = "mean", dist, distarg, p0 = 0, distbounds = c(-Inf, Inf))

popmean(dist, distarg, p0 = 0, distbounds = c(-Inf, Inf))

popsd(dist, distarg, p0 = 0, distbounds = c(-Inf, Inf))

popvar(dist, distarg, p0 = 0, distbounds = c(-Inf, Inf))

popcvar(dist, distarg, p0 = 0, distbounds = c(-Inf, Inf))

popskew(dist, distarg, p0 = 0, distbounds = c(-Inf, Inf))

popkurt(dist, distarg, p0 = 0, distbounds = c(-Inf, Inf))
}
\arguments{
\item{stat}{character; statistic to compute — one of \code{"mean"},
\code{"sd"}, \code{"var"}, \code{"cvar"}, \code{"skew"}, \code{"kurt"}}

\item{dist}{character; distribution name (e.g. \code{"norm"},
\code{"paretoII"})}

\item{distarg}{list of distribution arguments}

\item{p0}{numeric; probability zero (default 0)}

\item{distbounds}{numeric vector of length 2; distribution bounds (default
\code{c(-Inf, Inf)})}
}
\value{
scalar numeric
}
\description{
Computes theoretical descriptive statistics of a distribution.
}
\examples{

library(CoSMoS)

populationstat("mean", "norm", list(mean = 2, sd = 1))
populationstat("sd",   "norm", list(mean = 2, sd = 1))
populationstat("var",  "norm", list(mean = 2, sd = 1))
populationstat("cvar", "norm", list(mean = 2, sd = 1))
populationstat("skew", "norm", list(mean = 2, sd = 1))
populationstat("kurt", "norm", list(mean = 2, sd = 1))

}
\seealso{
\code{\link{moments}}, \code{\link{checkTS}}
}
\keyword{internal}
