% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-internal.R
\name{DHMgenSim}
\alias{DHMgenSim}
\title{Simulate one realisation using DHM circulant-embedding}
\usage{
DHMgenSim(Sj, rn = NULL)
}
\arguments{
\item{Sj}{numeric vector of spectral values from \code{DHMgenSj}}

\item{rn}{optional vector of standard normal random numbers of length
\code{2 * length(Sj) - 2}; generated internally if \code{NULL}}
}
\value{
numeric vector of length \code{length(Sj) - 1}
}
\description{
Simulates one realisation of a stationary Gaussian process from precomputed
spectral values \eqn{S_j} via the Dietrich-Newsam-Haskard algorithm.
}
\keyword{internal}
