% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/actf.R
\name{acti}
\alias{acti}
\title{ACTI — autocorrelation transformation integral function}
\usage{
acti(x, y, dist, distarg, rhoz, p0)
}
\arguments{
\item{x}{x-plane value}

\item{y}{y-plane value}

\item{dist}{distribution name}

\item{distarg}{a list of distribution arguments}

\item{rhoz}{Gaussian correlation}

\item{p0}{probability of zero values}
}
\value{
numeric scalar — integrand value at (x, y)
}
\description{
Expression supplied to the double integral inside \code{\link{actpnts}}.
}
\examples{
acti(1, -1, "norm", list(mean = 0, sd = 1), .3, 0)

}
\seealso{
\code{\link{actpnts}}
}
\keyword{internal}
